
import Tuple;
import Dimensions;
import Circle;
import Rectangle;
import Ball;
import Paddle;
import Target;

import java.awt.Color;
import java.awt.event.KeyListener;
import java.awt.event.MouseMotionListener;

public class Breakout extends Theater
   {
   /*
   Define a theater to represent a Breakout game.
   */

   //this game has a stage with some dimensions
   private int stageWidth, stageHeight;

   public Breakout(Dimensions dimensions, String soundDirectory)
      {
      /*
      Initialize this Breakout game.
      */

      super(dimensions, soundDirectory);

      createOrchestra();

      Dimensions stageDimensions = getStageDimensions();
      stageWidth = (int) stageDimensions.getWidth();
      stageHeight = (int) stageDimensions.getHeight();

      createActors();
      }

   public void start()
      {
      /*
      Start the game.
      */

      super.start();
      playSound("flourish.au");
      }

   protected void createOrchestra()
      {
      /*
      Give the orchestra a Breakout repertoire.
      */

      addSound("flourish.au");
      addSound("open.au");
      addSound("fire1.au");
      addSound("fire2.au");
      }

   protected void createActors()
      {
      /*
      Create the Breakout troupe of actors.
      */

      createBalls();
      createPaddle();
      createTargets();
      }

   protected void createBalls()
      {
      /*
      Create the lead Breakout actors---bouncing balls.
      */

      addActor(new Ball(this,
         new Circle(new Tuple(100, 100), 7),
         new Tuple(5, 7), Color.red));
      /*...for testing more complex games...
      addActor(new Ball(this,
         new Circle(new Tuple(stageWidth - 100, stageHeight - 100), 7),
         new Tuple(-7, 5), Color.pink));
      addActor(new Ball(this,
         new Circle(new Tuple(stageWidth - 100, 100), 7),
         new Tuple(-5, 7), Color.yellow));
      */
      }

   protected void createPaddle()
      {
      /*
      Create the user-controlled Breakout paddle.
      The user can move it with the arrow keys or with the mouse.
      */

      Paddle paddle = new Paddle(
         new Rectangle(new Tuple(stageWidth/2, stageHeight/2),
         new Dimensions(10, 50)), Color.blue);
         //alternate version for testing
         //new Rectangle(new Tuple(stageWidth/2, stageHeight/2),
         //new Dimensions(stageWidth/4, stageHeight/4)), Color.blue);

      addActor(paddle);

      //let the paddle pay attention to the keyboard and mouse
      this.addKeyListener((KeyListener) paddle);
      this.addMouseMotionListener((MouseMotionListener) paddle);
      this.requestFocus();
      }

   protected void createTargets()
      {
      /*
      Create four columns of differently colored Breakout targets,
      two on either side of the stage.
      */

      int targetWidth = 10;
      int targetHeight = 30;
      int ySpacing = (int) (1.5 * targetHeight);
      int targetsPerColumn = stageHeight / ySpacing;
      int[] xPositions = { targetWidth, 3 * targetWidth,
         stageWidth - 4 * targetWidth, stageWidth - 2 * targetWidth };
      int numberOfColumns = xPositions.length;
      Dimensions targetDimensions =
         new Dimensions(targetWidth, targetHeight);
      Color[] targetColor =
         { Color.green, Color.green, Color.yellow, Color.yellow };

      for (int index1 = 0; index1 < numberOfColumns; index1++)
         for (int index2 = 0; index2 < targetsPerColumn; index2++)
            addActor(new Target(
               new Rectangle(new Tuple(xPositions[index1],
               ySpacing * index2), targetDimensions),
               targetColor[index1]));
      }

   public static void main(String[] parameters)
      {
      /*
      Create and start a Breakout game.
      */

      (new Breakout(new Dimensions(400, 400), "BreakoutSounds")).start();
      }
   }
