
import Tuple;

public class Line
   {
   /*
   Define an object to hold a line.
   */

   //this line has a slope and a y-intercept
   private double slope, yIntercept;

   //this line may be horizontal or vertical, or may be just a point
   private boolean isHorizontal, isVertical, isPoint;

   //this line passes through two points
   private Tuple point1, point2;

   public Line(Tuple point1, Tuple point2)
      {
      /*
      Initialize this line to pass through the given points.
      */

      this.point1 = new Tuple(point1.getX(), point1.getY());
      this.point2 = new Tuple(point2.getX(), point2.getY());
      setConstants();
      }

   public final boolean isVertical()
      {
      /*
      Report whether this line is vertical.
      */

      return isVertical;
      }

   public final boolean isHorizontal()
      {
      /*
      Report whether this line is horizontal.
      */

      return isHorizontal;
      }

   public final boolean isPoint()
      {
      /*
      Report whether this line is really just a point.
      */

      return isPoint;
      }

   public final double getSlope()
      {
      /*
      Report this line's slope.
      */

      return slope;
      }

   public final double getYIntercept()
      {
      /*
      Report this line's y intercept.
      */

      return yIntercept;
      }

   public final Tuple getPoint()
      {
      /*
      Report a point on this line.
      */

      return new Tuple(point1.getX(), point1.getY());
      }

   public final boolean isParallelTo(Line line)
      {
      /*
      Report whether this line is parallel to the given one.
      */

      return ((this.isVertical() && line.isVertical()) ||
         ((this.getSlope() != 0) &&
         (this.getSlope() == line.getSlope())));
      }

   public boolean contains(Tuple point)
      {
      /*
      Report whether this line contains the given point.
      */

      if (this.isPoint() && (point.getX() == this.getPoint().getX())
         && (point.getY() == this.getPoint().getY()))
         return true;

      if (this.isVertical() && (point.getX() == this.getPoint().getX()))
         return true;

      double yValue = getSlope() * point.getX() + getYIntercept();

      return (point.getY() == yValue);
      }

   public Tuple intersects(Line line)
      {
      /*
      Report this line's intersection with the given one,
      or null if they don't intersect.
      */

      if (this.isPoint())
         {
         Tuple point = this.getPoint();
         if (line.contains(point))
            return point;
         else
            return null;
         }

      if (line.isPoint())
         {
         Tuple point = line.getPoint();
         if (this.contains(point))
            return point;
         else
            return null;
         }

      if (this.isParallelTo(line))
            return null;

      //at this point, neither line is a point nor are they parallel
      double x = 0; double y = 0;

      if (!this.isVertical() && line.isVertical())
         {
         x = line.getPoint().getX();
         y = this.getSlope() * x + this.getYIntercept();
         }

      if (this.isVertical() && !line.isVertical())
         {
         x = this.getPoint().getX();
         y = line.getSlope() * x + line.getYIntercept();
         }

      if (!this.isVertical() && !line.isVertical())
         {
         double slopeDifference = this.getSlope() - line.getSlope();
         x = (line.getYIntercept() - this.getYIntercept()) /
            slopeDifference;
         y = (this.getSlope() * line.getYIntercept() -
            line.getSlope() * this.getYIntercept()) / slopeDifference;
         }

      return new Tuple(x, y);
      }

   private final void setConstants()
      {
      /*
      Calculate this line's slope and y intercept.
      */

      double xRange = point1.getX() - point2.getX();
      double yRange = point1.getY() - point2.getY();

      isPoint = (xRange == 0) && (yRange == 0);
      isVertical = (xRange == 0) && (yRange != 0);
      isHorizontal = (xRange != 0) && (yRange == 0);

      slope = 0;
      if ((xRange != 0) && (yRange != 0))
         slope = yRange / xRange;

      yIntercept = 0;
      if (xRange != 0)
         yIntercept = (point1.getX() * point2.getY() -
            point2.getX() * point1.getY()) / xRange;
      }

   public String toString()
      {
      /*
      Report this line's string representation.
      */

      return point1.toString() + " " + point1.toString() +
         " slope = " + slope + " intercept = " + yIntercept;
      }
   }
