
import java.util.Vector;
import java.applet.Applet;
import java.applet.AudioClip;
import java.net.URL;
import java.net.MalformedURLException;

public class Orchestra
   {
   /*
   Define an ohject to represent and play a set of sounds.
   */

   //this orchestra's repertoire is a set of sounds and their names
   private Vector sounds, soundNames;

   //those sounds (as files) live in a directory
   private final String soundDirectory;

   public Orchestra(String soundDirectory)
      {
      /*
      Initialize this orchestra to get their sounds
      from the given directory.
      */

      sounds = new Vector(); soundNames = new Vector();
      this.soundDirectory = soundDirectory;
      }

   public void addSound(String soundFileName)
      {
      /*
      Add the named sound to this orchestra's repertoire.
      */

      if (soundFileName != null)
         {
         soundNames.addElement(soundFileName);
         sounds.addElement(loadSound(soundFileName));
         }
      }

   public void play(String soundName)
      {
      /*
      Play the named sound, if it's in the repertoire.
      */

      //make sure the sound's name isn't null
      if (soundName == null)
         return;

      //find the named sound
      int index = 0; boolean found = false;
      while ((index < sounds.size()) && !found)
         {
         if (soundName.equals((String) soundNames.elementAt(index)))
            found = true;
         else
            index++;
         }

      //make sure the orchestra knows the sound
      if (!found)
         return;

      //play it
      AudioClip sound = (AudioClip) sounds.elementAt(index);
      if (sound != null)
         sound.play();
      }

   private final AudioClip loadSound(String soundFileName)
      {
      /*
      Try to load the given sound from the sound directory.
      */

      String currentDirectory = System.getProperty("user.dir");
      String fileSeparator = System.getProperty("file.separator");
      String soundFile = currentDirectory + fileSeparator +
         soundDirectory + fileSeparator + soundFileName;

      //try to fetch the given sound file
      URL soundFileURL = null;
      try { soundFileURL = new URL("file:" + soundFile); }
      catch (MalformedURLException ignored) {}

      return Applet.newAudioClip(soundFileURL);
      }
   }
