
import Dimensions;
import CollisionDetector;
import Clock;
import ClockObservable;
import Actor;
import Troupe;
import Stage;
import Orchestra;

import java.awt.BorderLayout;
import javax.swing.JFrame;

public abstract class Theater extends JFrame
   implements ClockObservable
   {
   /*
   Define a class to represent a stage, some actors, an orchestra,
   and a clock.
   */

   //this theater has a stage, a band, a troupe of actors, and a clock
   private Stage stage;
   private Orchestra band;
   private Troupe troupe;
   private Clock clock;

   //this theater has a collision detector
   private CollisionDetector collisionDetector;

   //this theater's stage has dimensions
   private Dimensions stageDimensions;

   //this theater's stage has a polygonal perimeter
   private Polygon perimeter;

   //all theaters operate at a certain speed
   private final static int TICKS_PER_SECOND = 30;

   public Theater(Dimensions dimensions, String soundDirectory)
      {
      /*
      Initialize this theater.
      */

      createStage(dimensions);

      troupe = new Troupe();

      perimeter = createPerimeter();

      collisionDetector = new CollisionDetector(this);

      int width = (int) dimensions.getWidth();
      int height = (int) dimensions.getHeight();
      this.setSize(width, height);
      this.getContentPane().setLayout(new BorderLayout());
      this.getContentPane().add(stage, BorderLayout.CENTER);
      this.setVisible(true);

      if (soundDirectory != null)
         band = new Orchestra(soundDirectory);
      }

   public final Polygon getPerimeter()
      {
      /*
      Report this theater's stage's perimeter.
      */

      return perimeter;
      }

   private final Polygon createPerimeter()
      {
      /*
      Report a polygon representing the stage's rectangular perimeter
      in clockwise order: left, top, right, bottom.
      */

      double left = 0; double top = 0;
      double right = stageDimensions.getWidth();
      double bottom = stageDimensions.getHeight();

      Tuple[] points = { new Tuple(left, bottom), new Tuple(left, top),
         new Tuple(right, top), new Tuple(right, bottom) };

      return new Polygon(points);
      }

   public void tick()
      {
      /*
      Update this theater.
      */

      //ask the actors to act
      for (int index = 0; index < troupe.getSize(); index++)
         troupe.getActorAt(index).act();

      //check whether any of them collide
      collisionDetector.detectCollisions();

      //remove any that have been destroyed
      troupe.updateTroupe();

      //draw the actors in their new positions
      stage.repaint();
      }

   public Dimensions getStageDimensions()
      {
      /*
      Report this theater's stage's dimensions.
      */

      double width = stageDimensions.getWidth();
      double height = stageDimensions.getHeight();

      return new Dimensions(width, height);
      }

   public Troupe getTroupe()
      {
      /*
      Report this theater's troupe of actors.
      */

      return troupe;
      }

   public void start()
      {
      /*
      Start the play.
      */

      clock = new Clock(this, Theater.TICKS_PER_SECOND);
      clock.start();
      }

   public void stop()
      {
      /*
      Stop the play.
      */

      clock.stop();
      }

   public void removeActor(Actor actor)
      {
      /*
      Remove the named actor from the stage.
      */

      troupe.removeActor(actor);
      }

   public void playSound(String soundName)
      {
      /*
      Play the named sound.
      */

      if (band != null)
         band.play(soundName);
      }

   public void addSound(String soundName)
      {
      /*
      Add the named sound to the orchestra's repertoire.
      */

      if (band != null)
         band.addSound(soundName);
      }

   protected void addActor(Actor actor)
      {
      /*
      Add the given actor to the troupe of actors.
      */

      troupe.addActor(actor);
      }

   protected void createStage(Dimensions dimensions)
      {
      /*
      Create a stage.
      */

      int width = (int) dimensions.getWidth();
      int height = (int) dimensions.getHeight();

      //NOTE: dimension corrections needed due to java's implementation
      //on microsoft windows machines
      stageDimensions = new Dimensions(width - 10, height - 30);
      stage = new Stage(this);
      }
   }
